@@ -26,7 +26,6 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <poll.h>
-#include <siphash.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -64,6 +63,7 @@ static struct repo_read {
 	int *temp_fds;
 	int session_fd;
 	struct gotd_imsgev session_iev;
+	int refs_listed;
 } repo_read;
 
 static struct repo_read_client {
@@ -114,7 +114,7 @@ send_symref(struct got_reference *symref, struct got_object_id *target_id,
 	memset(&isymref, 0, sizeof(isymref));
 	isymref.name_len = strlen(refname);
 	isymref.target_len = strlen(target);
-	memcpy(isymref.target_id, target_id->sha1, sizeof(isymref.target_id));
+	memcpy(isymref.target_id, target_id->hash, sizeof(isymref.target_id));
 
 	len = sizeof(isymref) + isymref.name_len + isymref.target_len;
 	if (len > MAX_IMSGSIZE - IMSG_HEADER_SIZE) {
@@ -184,7 +184,7 @@ send_peeled_tag_ref(struct got_reference *ref, struct got_object *obj,
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1) {
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1) {
 		err = got_error_from_errno("imsg_add REF");
 		goto done;
 	}
@@ -232,7 +232,7 @@ send_ref(struct got_reference *ref, struct imsgbuf *ibuf)
 	}
 
 	/* Keep in sync with struct gotd_imsg_ref definition. */
-	if (imsg_add(wbuf, id->sha1, SHA1_DIGEST_LENGTH) == -1)
+	if (imsg_add(wbuf, id->hash, SHA1_DIGEST_LENGTH) == -1)
 		return got_error_from_errno("imsg_add REF");
 	if (imsg_add(wbuf, &namelen, sizeof(namelen)) == -1)
 		return got_error_from_errno("imsg_add REF");
@@ -260,7 +260,6 @@ list_refs(struct imsg *imsg)
 	struct repo_read_client *client = &repo_read_client;
 	struct got_reflist_head refs;
 	struct got_reflist_entry *re;
-	struct gotd_imsg_list_refs_internal ireq;
 	size_t datalen;
 	struct gotd_imsg_reflist irefs;
 	struct imsgbuf ibuf;
@@ -274,17 +273,15 @@ list_refs(struct imsg *imsg)
 		return got_error(GOT_ERR_PRIVSEP_NO_FD);
 
 	datalen = imsg->hdr.len - IMSG_HEADER_SIZE;
-	if (datalen != sizeof(ireq))
+	if (datalen != 0)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
-	memcpy(&ireq, imsg->data, sizeof(ireq));
 
-	if (ireq.client_id == 0)
-		return got_error(GOT_ERR_CLIENT_ID);
-	if (client->id != 0) {
+	if (repo_read.refs_listed) {
 		return got_error_msg(GOT_ERR_CLIENT_ID,
 		    "duplicate list-refs request");
 	}
-	client->id = ireq.client_id;
+	repo_read.refs_listed = 1;
+
 	client->fd = client_fd;
 
 	imsg_init(&ibuf, client_fd);
@@ -417,10 +414,10 @@ recv_want(struct imsg *imsg)
 	memcpy(&iwant, imsg->data, sizeof(iwant));
 
 	memset(&id, 0, sizeof(id));
-	memcpy(id.sha1, iwant.object_id, SHA1_DIGEST_LENGTH);
+	memcpy(id.hash, iwant.object_id, SHA1_DIGEST_LENGTH);
 
 	if (log_getverbose() > 0 &&
-	    got_sha1_digest_to_str(id.sha1, hex, sizeof(hex)))
+	    got_object_id_hex(&id, hex, sizeof(hex)))
 		log_debug("client wants %s", hex);
 
 	imsg_init(&ibuf, client->fd);
@@ -462,10 +459,10 @@ recv_have(struct imsg *imsg)
 	memcpy(&ihave, imsg->data, sizeof(ihave));
 
 	memset(&id, 0, sizeof(id));
-	memcpy(id.sha1, ihave.object_id, SHA1_DIGEST_LENGTH);
+	memcpy(id.hash, ihave.object_id, SHA1_DIGEST_LENGTH);
 
 	if (log_getverbose() > 0 &&
-	    got_sha1_digest_to_str(id.sha1, hex, sizeof(hex)))
+	    got_object_id_hex(&id, hex, sizeof(hex)))
 		log_debug("client has %s", hex);
 
 	imsg_init(&ibuf, client->fd);
@@ -580,15 +577,13 @@ static const struct got_error *
 receive_pack_pipe(struct imsg *imsg, struct gotd_imsgev *iev)
 {
 	struct repo_read_client *client = &repo_read_client;
-	struct gotd_imsg_packfile_pipe ireq;
 	size_t datalen;
 
 	log_debug("receiving pack pipe descriptor");
 
 	datalen = imsg->hdr.len - IMSG_HEADER_SIZE;
-	if (datalen != sizeof(ireq))
+	if (datalen != 0)
 		return got_error(GOT_ERR_PRIVSEP_LEN);
-	memcpy(&ireq, imsg->data, sizeof(ireq));
 
 	if (client->pack_pipe != -1)
 		return got_error(GOT_ERR_PRIVSEP_MSG);
@@ -605,9 +600,8 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 {
 	const struct got_error *err = NULL;
 	struct repo_read_client *client = &repo_read_client;
-	struct gotd_imsg_packfile_done idone;
-	uint8_t packsha1[SHA1_DIGEST_LENGTH];
-	char hex[SHA1_DIGEST_STRING_LENGTH];
+	struct got_object_id packhash;
+	char hex[GOT_HASH_DIGEST_STRING_MAXLEN];
 	FILE *delta_cache = NULL;
 	struct imsgbuf ibuf;
 	struct repo_read_pack_progress_arg pa;
@@ -647,7 +641,7 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 	if (err)
 		goto done;
 
-	err = got_pack_create(packsha1, client->pack_pipe, delta_cache,
+	err = got_pack_create(&packhash, client->pack_pipe, delta_cache,
 	    have_ids.ids, have_ids.nids, want_ids.ids, want_ids.nids,
 	    repo_read.repo, 0, 1, 0, pack_progress, &pa, &rl,
 	    check_cancelled, NULL);
@@ -655,13 +649,12 @@ send_packfile(struct imsg *imsg, struct gotd_imsgev *iev)
 		goto done;
 
 	if (log_getverbose() > 0 &&
-	    got_sha1_digest_to_str(packsha1, hex, sizeof(hex)))
+	    got_hash_digest_to_str(packhash.hash, hex, sizeof(hex),
+	    packhash.algo))
 		log_debug("sent pack-%s.pack", hex);
 
-	memset(&idone, 0, sizeof(idone));
-	idone.client_id = client->id;
 	if (gotd_imsg_compose_event(iev, GOTD_IMSG_PACKFILE_DONE,
-	    PROC_REPO_READ, -1, &idone, sizeof(idone)) == -1)
+	    PROC_REPO_READ, -1, NULL, 0) == -1)
 		err = got_error_from_errno("imsg compose PACKFILE_DONE");
 done:
 	if (client->delta_cache_fd != -1 &&
@@ -709,7 +702,7 @@ repo_read_dispatch_session(int fd, short event, void *arg)
 			break;
 
 		if (imsg.hdr.type != GOTD_IMSG_LIST_REFS_INTERNAL &&
-		    client->id == 0) {
+		    !repo_read.refs_listed) {
 			err = got_error(GOT_ERR_PRIVSEP_MSG);
 			break;
 		}
@@ -893,6 +886,11 @@ repo_read_main(const char *title, const char *repo_path,
 		    "bare git repository required");
 		goto done;
 	}
+	if (got_repo_get_object_format(repo_read.repo) != GOT_HASH_SHA1) {
+		err = got_error_msg(GOT_ERR_NOT_IMPL,
+		    "sha256 object IDs unsupported in network protocol");
+		goto done;
+	}
 
 	got_repo_temp_fds_set(repo_read.repo, temp_fds);
 
@@ -925,7 +923,7 @@ repo_read_shutdown(void)
 {
 	struct repo_read_client *client = &repo_read_client;
 
-	log_debug("shutting down");
+	log_debug("%s: shutting down", repo_read.title);
 
 	if (client->have_ids)
 		got_object_idset_free(client->have_ids);
