@@ -523,23 +523,23 @@ forward_notification(struct gotd_session_client *client, struct imsg *imsg)
 		goto done;
 	}
 	if (notif->action == GOTD_NOTIF_ACTION_CREATED) {
-		if (memcmp(notif->new_id.sha1, icontent.new_id,
-		    SHA1_DIGEST_LENGTH) != 0) {
+		if (memcmp(&notif->new_id, &icontent.new_id,
+		    sizeof(notif->new_id)) != 0) {
 			err = got_error_msg(GOT_ERR_PRIVSEP_MSG,
 			    "received notification content for unknown event");
 			goto done;
 		}
 	} else if (notif->action == GOTD_NOTIF_ACTION_REMOVED) {
-		if (memcmp(notif->old_id.sha1, icontent.old_id,
-		    SHA1_DIGEST_LENGTH) != 0) {
+		if (memcmp(&notif->old_id, &icontent.old_id,
+		    sizeof(notif->old_id)) != 0) {
 			err = got_error_msg(GOT_ERR_PRIVSEP_MSG,
 			    "received notification content for unknown event");
 			goto done;
 		}
-	} else if (memcmp(notif->old_id.sha1, icontent.old_id,
-	    SHA1_DIGEST_LENGTH) != 0 ||
-	    memcmp(notif->new_id.sha1, icontent.new_id,
-	    SHA1_DIGEST_LENGTH) != 0) {
+	} else if (memcmp(&notif->old_id, &icontent.old_id,
+	    sizeof(notif->old_id)) != 0 ||
+	    memcmp(&notif->new_id, &icontent.new_id,
+	    sizeof(notif->old_id)) != 0) {
 		err = got_error_msg(GOT_ERR_PRIVSEP_MSG,
 		    "received notification content for unknown event");
 		goto done;
@@ -696,8 +696,10 @@ update_ref(int *shut, struct gotd_session_client *client,
 
 	log_debug("updating ref %s for uid %d", refname, client->euid);
 
-	memcpy(old_id.sha1, iref.old_id, SHA1_DIGEST_LENGTH);
-	memcpy(new_id.sha1, iref.new_id, SHA1_DIGEST_LENGTH);
+	memset(&old_id, 0, sizeof(old_id));
+	memcpy(old_id.hash, iref.old_id, SHA1_DIGEST_LENGTH);
+	memset(&new_id, 0, sizeof(new_id));
+	memcpy(new_id.hash, iref.new_id, SHA1_DIGEST_LENGTH);
 	err = got_repo_find_object_id(iref.delete_ref ? &old_id : &new_id,
 	    repo);
 	if (err)
@@ -1700,6 +1702,11 @@ session_write_main(const char *title, const char *repo_path,
 		    "bare git repository required");
 		goto done;
 	}
+	if (got_repo_get_object_format(gotd_session.repo) != GOT_HASH_SHA1) {
+		err = got_error_msg(GOT_ERR_NOT_IMPL,
+		    "sha256 object IDs unsupported in network protocol");
+		goto done;
+	}
 
 	got_repo_temp_fds_set(gotd_session.repo, temp_fds);
 
