@@ -104,6 +104,8 @@ clear_meta(struct got_pack_meta *meta)
 	free(meta->base_obj_id);
 	meta->base_obj_id = NULL;
 	meta->reused_delta_offset = 0;
+	got_deltify_free(meta->dtab);
+	meta->dtab = NULL;
 }
 
 static void
@@ -113,6 +115,7 @@ free_nmeta(struct got_pack_meta **meta, int nmeta)
 
 	for (i = 0; i < nmeta; i++)
 		clear_meta(meta[i]);
+
 	free(meta);
 }
 
@@ -688,10 +691,6 @@ pick_deltas(struct got_pack_meta **meta, int nmeta, int ncolored,
 		raw = NULL;
 	}
 done:
-	for (i = MAX(0, nmeta - max_base_candidates); i < nmeta; i++) {
-		got_deltify_free(meta[i]->dtab);
-		meta[i]->dtab = NULL;
-	}
 	if (raw)
 		got_object_raw_close(raw);
 	if (base_raw)
