@@ -53,9 +53,9 @@
 #include "got_object.h"
 
 #include "got_lib_delta.h"
+#include "got_lib_hash.h"
 #include "got_lib_object.h"
 #include "got_lib_privsep.h"
-#include "got_lib_hash.h"
 
 struct imsgbuf ibuf;
 
@@ -182,7 +182,6 @@ filexbit(const char *line)
 static const struct got_error *
 blobid(const char *line, char **blob, int git)
 {
-	uint8_t digest[SHA1_DIGEST_LENGTH];
 	size_t len;
 
 	*blob = NULL;
@@ -191,7 +190,11 @@ blobid(const char *line, char **blob, int git)
 	if ((*blob = strndup(line, len)) == NULL)
 		return got_error_from_errno("strndup");
 
-	if (!git && !got_parse_hash_digest(digest, *blob, GOT_HASH_SHA1)) {
+	if (git)
+		return NULL;
+
+	if (len != got_hash_digest_string_length(GOT_HASH_SHA1) - 1 &&
+	    len != got_hash_digest_string_length(GOT_HASH_SHA256) - 1) {
 		/* silently ignore invalid blob ids */
 		free(*blob);
 		*blob = NULL;
@@ -692,8 +695,8 @@ main(int argc, char **argv)
 		goto done;
 	}
 	err = got_privsep_flush_imsg(&ibuf);
-done:
 	imsg_free(&imsg);
+done:
 	if (fd != -1 && close(fd) == -1 && err == NULL)
 		err = got_error_from_errno("close");
 	if (fp != NULL && fclose(fp) == EOF && err == NULL)
